# Generated by Django 3.2 on 2021-04-09 06:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_db_lock', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lock',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='lock',
            name='worker_name',
            field=models.CharField(max_length=128, verbose_name='Worker(Locker) Name'),
        ),
    ]
